"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
const vscode_1 = require("vscode");
const nls = __importStar(require("vscode-nls"));
const interfaces_1 = require("../interfaces");
const default_1 = require("./default");
const localize = nls.config({ messageFormat: nls.MessageFormat.both })(__filename);
const I18N = {
    document: localize(0, null),
    github: localize(1, null),
    whenToUse: localize(2, null),
    type: localize(3, null),
    optionalValue: localize(4, null),
    defaultValue: localize(5, null),
    library: localize(6, null),
};
exports.CONFIG = {
    language: 'en-US',
    hover: true,
    inlineTemplate: true,
    isAntd: true,
    isAlain: true,
};
exports.RESOURCES = [];
exports.DIRECTIVE_NAMES = [];
function notNull(value, defaultValue = '') {
    if (value == null)
        return defaultValue;
    return value;
}
function getPure(value) {
    return value.replace(/nz([A-Z])/g, (_raw, val) => val.toLowerCase());
}
function getFullDoc(item) {
    if (item.doc.startsWith('http'))
        return item.doc;
    if (item.lib === 'ng-zorro-antd') {
        // TODO: Muse be release version
        return (exports.CONFIG.language === 'zh-CN' ? 'https://ng-zorro.gitee.io' : 'https://ng.ant.design') + item.doc;
        // return `https://ng-zorro-master.netlify.com` + item.doc;
    }
    return (exports.CONFIG.language === 'zh-CN' ? 'https://ng-alain.com' : 'https://ng-alain-doc.surge.sh') + item.doc;
}
function genComponentMarkdown(item) {
    if (item == null)
        return '';
    const rows = [exports.CONFIG.isAlain ? `**${I18N.library}** ${item.lib}` : null, `**${item.title}**`, item.description];
    if (item.whenToUse && item.whenToUse.trim().length > 0) {
        rows.push(`**${I18N.whenToUse}**`);
        rows.push(item.whenToUse);
    }
    if (item.doc) {
        rows.push(`[${I18N.document}](${getFullDoc(item)})${item.github ? ` － [${I18N.github}](${item.github})` : ''}`);
    }
    return rows.filter((w) => !!w).join('\n\n');
}
function genPropertyMarkdown(property) {
    if (property == null)
        return '';
    const rows = [];
    if (property.typeDefinition && Array.isArray(property.typeDefinition) && property.typeDefinition.length > 0) {
        rows.push(`**${I18N.optionalValue}** ${property.typeDefinition.map((i) => '`' + i.label + '`').join(', ')}`);
    }
    else if (property.typeRaw.length > 0) {
        rows.push(`**${I18N.type}** ${property.typeRaw}`);
    }
    if (property.default.length > 0) {
        rows.push(`**${I18N.defaultValue}** ${property.default}`);
    }
    rows.push(property.description);
    return rows.join('\n\n');
}
function fixProperty(p) {
    p.pureName = getPure(p.name);
    p.default = notNull(p.default, '-');
    if (p.default === '-')
        p.default = '';
    p.description = default_1.LANG[p.description];
    p.isInputBoolean = notNull(p.isInputBoolean, true);
    p.type = notNull(p.type, 'string');
    p.typeRaw = notNull(p.typeRaw, '').replace(/｜/g, '|');
    p.typeDefinition = notNull(p.typeDefinition, []);
    if (Array.isArray(p.typeDefinition)) {
        p.typeDefinition = p.typeDefinition.map((item) => (typeof item === 'string' ? { value: item, label: item } : item));
        p.typeDefinitionSnippetStr = p.typeDefinition.map((i) => i.value).join(',');
    }
    else {
        if (typeof p.typeDefinition === 'object') {
            p.inputType = interfaces_1.InputAttrType.Complex;
        }
        p.typeDefinitionSnippetStr = '';
    }
    p._doc = new vscode_1.MarkdownString(genPropertyMarkdown(p));
}
function INIT(notifier) {
    return __awaiter(this, void 0, void 0, function* () {
        const cog = vscode_1.workspace.getConfiguration(interfaces_1.NAME);
        exports.CONFIG.language = cog.language;
        exports.CONFIG.hover = cog.hover;
        exports.CONFIG.inlineTemplate = cog.inlineTemplate;
        if (vscode_1.workspace.workspaceFolders != null && vscode_1.workspace.workspaceFolders.length > 0) {
            exports.CONFIG.isAntd = false;
            exports.CONFIG.isAlain = false;
            const packageJsonPath = path_1.join(vscode_1.workspace.workspaceFolders[0].uri.fsPath, 'package.json');
            if (!fs_1.existsSync(packageJsonPath)) {
                notifier.notify(`Not found package.json`);
                return null;
            }
            const packageJson = JSON.parse(fs_1.readFileSync(packageJsonPath).toString());
            const VERSIONS = [...Object.keys(packageJson.dependencies), ...Object.keys(packageJson.devDependencies)];
            if (~VERSIONS.indexOf(`ng-zorro-antd`)) {
                exports.CONFIG.isAntd = true;
            }
            if (~VERSIONS.indexOf(`@delon/theme`)) {
                exports.CONFIG.isAntd = true;
                exports.CONFIG.isAlain = true;
            }
        }
        const data = default_1.DATA;
        if (exports.CONFIG.isAntd) {
            exports.RESOURCES.push(...data.filter((i) => i.lib === 'ng-zorro-antd'));
        }
        if (exports.CONFIG.isAlain) {
            exports.RESOURCES.push(...data.filter((i) => i.lib !== 'ng-zorro-antd'));
        }
        exports.RESOURCES.forEach((i) => {
            i.description = default_1.LANG[i.description];
            i.whenToUse = default_1.LANG[i.whenToUse];
            i.doc = notNull(i.doc);
            i.title = default_1.LANG[i.title];
            i.type = notNull(i.type, 'component');
            i.snippet = notNull(i.snippet, '');
            if (i.snippet.length === 0) {
                i.snippet = i.type === 'component' ? `<__$1>$0</__>` : `<div __$1>$0</div>`;
            }
            i.snippet = i.snippet.replace(/__/g, i.selector);
            i.properties = notNull(i.properties, []);
            i.properties.forEach((p) => fixProperty(p));
            if (i.types) {
                Object.keys(i.types).forEach((key) => {
                    i.types[key].forEach((p) => fixProperty(p));
                });
            }
            if (~i.properties.findIndex((w) => w.name === i.selector)) {
                i.directiveNameIsOutput = true;
            }
            i._doc = new vscode_1.MarkdownString(genComponentMarkdown(i));
            exports.DIRECTIVE_NAMES.push(i.selector);
        });
    });
}
exports.INIT = INIT;
function isComponent(text) {
    return text.startsWith('nz-') || exports.DIRECTIVE_NAMES.indexOf(text) !== -1;
}
exports.isComponent = isComponent;
function getDirective(tag) {
    const dict = {};
    if (isComponent(tag.name)) {
        dict[tag.name] = first(tag.name);
    }
    Object.keys(tag.attributes)
        .filter((key) => exports.DIRECTIVE_NAMES.indexOf(tag.attributes[key].name) !== -1)
        .forEach((key) => {
        const keyValue = tag.attributes[key].name;
        if (dict[keyValue])
            return;
        dict[keyValue] = first(keyValue);
    });
    return Object.values(dict).filter((w) => !!w);
}
exports.getDirective = getDirective;
function query(type) {
    return [...exports.RESOURCES].filter((i) => i.type === type);
}
exports.query = query;
function first(selector) {
    return exports.RESOURCES.find((w) => w.selector === selector);
}
exports.first = first;

//# sourceMappingURL=index.js.map
